
import os
from pathlib import Path

from .execute_command import execute_command


def handle_test_command(base_package: str,
                        root_of_git_repository: str,
                        with_coverage: bool = False) -> None:
    test_command = 'pytest --color=yes'
    if with_coverage:
        cov_pkg = find_coverage_package(root_of_git_repository, base_package)
        test_command += ' --cov {}'.format(cov_pkg)
    test_dir = os.path.join(root_of_git_repository, 'tests')
    test_command += ' ' + test_dir
    execute_command(test_command)
    if with_coverage:
        execute_command('coverage html')
        print('Coverage report generated in htmlcov/ directory.')
        print('To view, open htmlcov/index.html in a web browser, or run:\n')
        print('    osewb browse coverage\n')


def find_coverage_package(root_of_git_repository: str, base_package: str) -> str:
    if is_workbench(root_of_git_repository):
        library_pkg = os.path.join(root_of_git_repository, base_package)
        if not Path(library_pkg).exists():
            print('library package not found')
            exit(1)
        return os.path.join(root_of_git_repository, base_package)
    else:  # Assume library with base package -> app / gui sub-packages
           # this assumption currently only holds true for ose-workbench-core
        app_pkg = os.path.join(root_of_git_repository, base_package, 'app')
        if not Path(app_pkg).exists():
            print('app package not found')
            exit(1)
        return app_pkg


def is_workbench(root_of_git_repository: str) -> bool:
    return Path(root_of_git_repository).joinpath('freecad').exists()



import os
from subprocess import PIPE, Popen
from typing import List


def execute_command(command: str,
                    env: dict = {},
                    exit_on_non_zero_code: bool = True) -> int:
    print('Executing the following command:\n')
    print('    {}\n'.format(command))
    environ = os.environ.copy()
    environ.update(env)
    with Popen(command, stdout=PIPE, bufsize=1, universal_newlines=True, shell=True, env=environ) as process:
        for line in process.stdout:
            print(line, end='')
    if process.returncode != 0 and exit_on_non_zero_code:
        exit(1)
    return process.returncode


def chain_commands(*commands: str,
                   env: dict = {},
                   exit_on_non_zero_code: bool = True) -> List[int]:
    return_codes = []
    for command in commands:
        return_code = execute_command(command, env=env,
                                      exit_on_non_zero_code=exit_on_non_zero_code)
        return_codes.append(return_code)
    return return_codes



import FreeCAD as App

from freecad.{{ base_package }}.icon import get_icon_path


class {{ name }}Command:
    """TODO: Fill in the blank: Command to ____."""

    NAME = '{{ name }}'

    def Activated(self):
        document = App.ActiveDocument
        if not document:
            document = App.newDocument()
        # TODO: Create some part
        document.recompute()

    def IsActive(self):
        return True

    def GetResources(self):
        # TODO: Fill-in icon name, MenuText, and ToolTip
        return {
            'Pixmap': get_icon_path(''),
            'MenuText': '',
            'ToolTip': ''
        }



import inspect
import os
import re
from os import listdir
from os.path import isfile, join

from sphinx.application import Sphinx
from sphinx.util.logging import getLogger

from osewb.find_base_package import find_root_of_git_repository

logger = getLogger(__name__)

# TODO: Make lint rule to enforce these naming conventions
class_pattern_template = r'ose[A-Za-z0-9]+\.{}\.[A-Z][A-Za-z]+'
part_class_pattern = re.compile(class_pattern_template.format('part'))
model_class_pattern = re.compile(class_pattern_template.format('model'))

icon_package_pattern = re.compile(r'freecad\.ose[A-Za-z0-9]+\.icon')

repo_root = find_root_of_git_repository()
if repo_root is None:
    print('Must be in git repository')
    exit(1)
screenshot_dir = os.path.join(repo_root, 'docs', '_static', 'screenshot')
img_path_template = os.path.join(screenshot_dir, '{}.png')


def process_docstring(app, what, name, obj, options, lines):
    if what == 'module' and icon_package_pattern.match(name):
        icon_directory = os.path.dirname(obj.__file__)
        icons = [f for f in listdir(icon_directory) if isfile(
            join(icon_directory, f)) and not f.endswith('.py')]
        lines.append('Icons')
        lines.append('-----')
        lines.append('')
        sorted_icons = sorted(icons)
        lines.append('.. list-table::')
        lines.append('   :header-rows: 1')
        lines.append('')
        lines.append('   * - Icon')
        lines.append('     - Filename')
        for icon in sorted_icons:
            lines.append('   * - :fcicon:`{} (md) <{}>`'.format(icon, icon))
            lines.append('     - ``{}``'.format(icon))
        lines.append('')
        lines.append('----')
        lines.append('')
    elif what == 'class':
        if part_class_pattern.match(name):
            class_name = obj.__name__
            if os.path.exists(img_path_template.format(class_name)):
                lines.append(
                    '.. image:: /_static/screenshot/{}.png'.format(class_name))
                lines.append('   :alt: {}'.format(class_name))
            else:
                logger.warning(
                    'Screenshot for part "{}" missing. Try running `osewb docs screenshot`.'.format(class_name))
        elif model_class_pattern.match(name) and name.endswith('Model'):
            lines.append('.. fc-custom-property-table::')


def setup(app: Sphinx) -> None:
    """Setup extension.

    :param app: application object controlling high-level functionality,
                such as the setup of extensions, event dispatching, and logging.
                See Also:
                https://www.sphinx-doc.org/en/master/extdev/appapi.html#sphinx.application.Sphinx
    """
    app.connect('autodoc-process-docstring', process_docstring)
    if not os.path.exists(screenshot_dir):
        logger.warning(
            'No screenshot directory detected.\nTo generate part screenshots, run:\n\n    osewb docs screenshot\n')


__all__ = ['setup']



from .execute_command import chain_commands


def handle_build_command() -> None:
    return_codes = chain_commands('osewb test --coverage',
                                  'osewb lint',
                                  'osewb docs',
                                  exit_on_non_zero_code=False)
    return_code_pairs = zip(('test', 'lint', 'docs'), return_codes)
    if any(return_codes):
        # TODO: There's some duplication between this and the lint command.
        #       see handle_lint_command.py
        print('Build failed with errors from the following command(s):')
        commands_with_errors = [pair[0]
                                for pair in return_code_pairs if pair[1]]
        print('    {}\n'.format(', '.join(commands_with_errors)))
        exit(1)
    else:
        print('Build passed.')



import os
from pathlib import Path

from .execute_command import execute_command


def handle_docs_command(base_package, root_of_git_repository) -> None:
    path_to_docs_dir = os.path.join(
        root_of_git_repository, 'docs')

    path_to_build_dir = os.path.join(
        path_to_docs_dir, '_build')
    remove_existing_directory_recursively(path_to_build_dir)

    path_to_library_package_sphinx_source_dir = os.path.join(
        path_to_docs_dir, base_package)
    remove_existing_directory_recursively(
        path_to_library_package_sphinx_source_dir)

    path_to_workbench_package_sphinx_source_dir = os.path.join(
        path_to_docs_dir, 'freecad')
    remove_existing_directory_recursively(
        path_to_workbench_package_sphinx_source_dir)

    execute_command(
        'cd {} && sphinx-build -W --keep-going --color . {}'.format(path_to_docs_dir, path_to_build_dir))

    print('To view, open docs/_build/index.html in a web browser, or run:\n')
    print('    osewb browse docs\n')


def remove_existing_directory_recursively(directory: str) -> None:
    if Path(directory).exists():
        execute_command('rm -rf {}'.format(directory))



from typing import Union

import Part
from osecore.app.model import Model
{%- if part and base_package %}
from {{ base_package }}.part import {{ part }}
{%- endif %}


class {{ name }}Model(Model):

    Type = 'OSE{{ name }}'

    def __init__(self, obj):
        super({{ name }}Model, self).__init__(obj)
        # TODO: Add custom properties
        # obj.addProperty('App::PropertyLength', 'PropertyName',
        #         'Base', 'Descriptive tooltip').PropertyName = 10.0

    def execute(self, obj):
        """
        Called on document recompute
        """
        {%- if part %}
        obj.Shape = {{ part }}.make()
        {%- else %}
        obj.Shape = Part.makeBox(10, 10, 10)
        {%- endif %}

    def __getstate__(self) -> Union[str, tuple]:
        """Execute when serializing and persisting the object.

        See Also:
            https://docs.python.org/3/library/pickle.html#object.__getstate__

        :return: state
        """
        return self.Type

    def __setstate__(self, state: str) -> None:
        """Execute when deserializing the object.

        See Also:
            https://docs.python.org/3/library/pickle.html#object.__setstate__

        :param state: state, in this case type of object.
        """
        if state:
            self.Type = state


import os
from typing import Optional, Union

from .check_for_executable_in_path import check_for_executable_in_path


def find_base_package() -> Optional[str]:
    """Find the base package in a workbench repository.

    Return None if not in a git repository,
    or no directory in the root of the repository starts with "ose".

    If multiple directories in the root of the repository start with "ose",
    then we return the first match.

    :return: Base package of workbench repository.
    """
    repo_root = find_root_of_git_repository()
    if repo_root is None:
        return None
    contents = os.listdir(repo_root)
    directories = [c for c in contents if os.path.isdir(
        os.path.join(repo_root, c)) and c.startswith('ose') and not c.endswith('egg-info')]
    if len(directories) == 0:
        print('No base package starting with "ose" found in repository.')
        return None
    elif len(directories) > 1:
        print('Multiple potential base packages starting with "ose" found:\n')
        print('    {}\n'.format(', '.join(directories)))
        print('Choosing first "{}" as base package.\n'.format(directories[0]))
    return directories[0]


def find_root_of_git_repository() -> Optional[str]:
    """Find the root of the current git repository.

    Returns None if there's an error, or not in a git repository.

    :return: path to root of git repository
    """
    return exec_git_command('git rev-parse --show-toplevel')


def find_git_user_name() -> Optional[str]:
    """Find the user name defined by git config.

    :return: Git user name
    """
    return exec_git_command('git config user.name')


def exec_git_command(git_command: str) -> Optional[str]:
    """Find the root of the current git repository.

    Returns None if there's an error, or not in a git repository.

    :param git_command: git command string
    :return: path to root of git repository
    """
    check_for_executable_in_path('git')
    pipe = os.popen(git_command)
    output = pipe.read().strip()
    if pipe.close() is not None:
        return None
    return output



import importlib
import inspect
import os
import re
from pathlib import Path
from typing import List

from isort import SortImports
from jinja2 import Environment, PackageLoader


def handle_make_component_command(base_package: str,
                                  root_of_git_repository: str,
                                  component: str,
                                  name: str,
                                  args: dict) -> None:
    # 1. Construct component package
    component_package_path = Path(os.path.join(
        root_of_git_repository,
        map_base_package_to_library_or_workbench_package(
            base_package, component),
        component
    ))
    component_package_path.mkdir(exist_ok=True)

    # 2. Construct new component sub-package
    #    Stop if it exists
    new_component_sub_package = '_' + camel_case_to_snake_case(name)
    new_component_sub_package_path = Path(
        component_package_path).joinpath(new_component_sub_package)
    if(new_component_sub_package_path.exists()):
        print('{} package already exists. Skipping creation.'.format(
            new_component_sub_package))
        return None
    new_component_sub_package_path.mkdir(exist_ok=True)

    # 3. Update init module of component package
    component_package_init_module_path = component_package_path.joinpath(
        '__init__.py')
    component_package_init_module_path.touch(exist_ok=True)
    component_name = map_name_to_component_name(name, component)
    with component_package_init_module_path.open('r+') as f:
        lines = f.readlines()
        text = ''.join(lines)
        white_space = '[\s\S]+'
        all_pattern = white_space.join(['__all__', '=', '(\[', '\])'])
        match = re.search(all_pattern, text)
        all_members = []
        if match:
            all_members = eval(match.group(1))
        next_all_string = build_next_all_string(
            all_members, component_name)
        if not match:
            text = next_all_string
        new_contents = re.sub(all_pattern, next_all_string, text)
        import_statement = 'from .{} import {}\n'.format(
            new_component_sub_package, component_name)
        new_contents = import_statement + '\n' + new_contents
        f.seek(0)
        f.write(new_contents)
    SortImports(component_package_init_module_path.resolve())

    # 4. Setup init module in component sub-package
    component_module_name = map_name_to_component_module_name(name, component)
    init_module_path = new_component_sub_package_path.joinpath(
        '__init__.py')
    init_module_path.touch(exist_ok=True)
    with init_module_path.open('a') as f:
        import_statement = 'from .{} import {}\n'.format(
            component_module_name, component_name)
        all_declaration = "__all__ = ['{}']".format(component_name)
        f.write('\n'.join([import_statement, all_declaration]))
    SortImports(init_module_path.resolve())

    # 5. Create new component module using template
    module_name = '{}.py'.format(component_module_name)
    component_module_path = new_component_sub_package_path.joinpath(
        module_name)
    component_module_existed_before = component_module_path.exists()
    component_module_path.touch(exist_ok=True)
    if component_module_existed_before:
        print('{} already exists. Skipping {} class creation.'.format(
            module_name, component))
        return None
    component_module_text = render_template(
        component, name, base_package, args)
    component_module_path.write_text(component_module_text)
    if component == 'model' and args['part']:
        del args['part']
        handle_make_component_command(base_package,
                                      root_of_git_repository,
                                      'part',
                                      name,
                                      args)


def build_next_all_string(all_members: List[str], component_name: str) -> str:
    next_all_members = all_members + [component_name]
    sorted_next_all_members = sorted(next_all_members)
    next_all_members_with_indentation = [
        "    '{}'".format(m) for m in sorted_next_all_members]
    next_all_members_string = ',\n'.join(
        next_all_members_with_indentation)
    return '__all__ = [\n{}\n]'.format(
        next_all_members_string)


def render_template(component: str, name: str, base_package: str, args: dict) -> str:
    env = Environment(
        loader=PackageLoader('osewb', 'templates'))
    template = env.get_template('{}.py'.format(component))
    template_args = get_component_template_args(component, name, args)
    return template.render(name=name, base_package=base_package, **template_args) + '\n'


def map_name_to_component_name(name: str, component: str) -> str:
    try:
        return {
            'part': name,
            'model': '{}Model'.format(name),
            'command': '{}Command'.format(name),
            'part_feature': 'create_{}'.format(name)
        }[component]
    except KeyError:
        print('No component named "{}"'.format(component))
        exit(1)


def map_name_to_component_module_name(name: str, component: str) -> str:
    try:
        return {
            'part': camel_case_to_snake_case(name),
            'model': '{}_model'.format(camel_case_to_snake_case(name)),
            'command': '{}_command'.format(camel_case_to_snake_case(name)),
            'part_feature': 'create_{}'.format(camel_case_to_snake_case(name))
        }[component]
    except KeyError:
        print('No component named "{}"'.format(component))
        exit(1)


def get_component_template_args(component: str, name: str, args: dict) -> dict:
    if component == 'model' and args['part']:
        return {
            'part': map_name_to_component_name(name, 'part')
        }
    if component == 'part_feature':
        return {
            'model': map_name_to_component_name(
                snake_case_to_camel_case(name), 'model')
        }
    return {}


def map_base_package_to_library_or_workbench_package(base_package: str,
                                                     component: str) -> str:
    library_package_components = ['part', 'model']
    workbench_package_components = ['command', 'part_feature']
    if component in library_package_components:
        return base_package
    elif component in workbench_package_components:
        return os.path.join('freecad', base_package)
    else:
        raise ValueError(
            'Component "{}" not in a library or workbench package.'.format(component))


def camel_case_to_snake_case(string: str) -> str:
    result = [string[0].lower()]
    for c in string[1:]:
        if c in 'ABCDEFGHIJKLMNOPQRSTUVWXYZ':
            result.append('_')
            result.append(c.lower())
        else:
            result.append(c)

    return ''.join(result)


def snake_case_to_camel_case(string: str) -> str:
    return string.replace('_', ' ').title().replace(' ', '')



import inspect

from sphinx.application import Sphinx


def process_docstring(app, what, name, obj, options, lines):
    if what == 'module':
        mutate_module_docstring_lines(obj, name, lines)

def mutate_module_docstring_lines(obj, name, lines):
    public_members = []
    if hasattr(obj, '__all__'):
        public_members = obj.__all__
    all_members = inspect.getmembers(obj)
    members = [
        (
            name + '.' + member[0],
            get_summary_line(inspect.getdoc(member[1]))
        )
        for member in all_members
        if not member[0].startswith('_')
        and member[0] in public_members
        and not inspect.isbuiltin(member[1])
        and (inspect.isclass(member[1]) or inspect.isfunction(member[1]))
    ]
    if len(members):
        lines.append('')
        lines.append('.. list-table::')
        lines.append('   :header-rows: 1')
        lines.append('')
        lines.append('   * - Name')
        lines.append('     - Description')
        lines.append('')
    for member, summary in members:
        lines.append('   * - :mod:`~{}`'.format(member))
        if summary:
            lines.append('     - ' + summary)
        else:
            lines.append('     - None')
        lines.append('')
    if len(members):
        lines.append('')
        lines.append('----')
        lines.append('')


def get_summary_line(docstring):
    if docstring is None:
        return None
    return docstring.splitlines()[0]


def setup(app: Sphinx) -> None:
    """Setup extension.

    :param app: application object controlling high-level functionality,
                such as the setup of extensions, event dispatching, and logging.
                See Also:
                https://www.sphinx-doc.org/en/master/extdev/appapi.html#sphinx.application.Sphinx
    """
    app.connect('autodoc-process-docstring', process_docstring)


__all__ = ['setup']



"""Package containing icons."""
import os


def get_icon_path(icon_filename: str) -> str:
    """Get the path to a given icon.

    :param icon_filename: icon filename (e.g. Box.svg)
    :type icon_filename: str
    :return: Path to icon file.
    :rtype: str
    """
    return os.path.join(os.path.dirname(__file__), icon_filename)


__all__ = ['get_icon_path']



import json
import os
import sys
from pathlib import Path
from typing import Optional


def handle_editor_config_command(root_of_git_repository: str,
                                 merge_workspace_settings: bool = False,
                                 overwrite_workspace_settings: bool = False) -> None:
    config = get_editor_config()
    workspace_settings = None
    if merge_workspace_settings or overwrite_workspace_settings:
        workspace_settings = get_vs_code_workspace_settings(
            root_of_git_repository)
        if merge_workspace_settings:
            if workspace_settings is None:
                print('No workspace settings found. Skipping merge.')
                workspace_settings = {}
            workspace_settings.update(config)
            config = workspace_settings

    # By default, VS Code uses 4 spaces for indentation
    # https://code.visualstudio.com/docs/editor/codebasics#_indentation
    stringified_config = json.dumps(config, indent=4, sort_keys=False)

    if overwrite_workspace_settings:
        workspace_settings_path = get_vs_code_workspace_settings_path(
            root_of_git_repository)
        question = 'Do you want to overwrite "{}"?'.format(
            workspace_settings_path)
        answer = query_yes_no(question, stringified_config)
        if answer is True:
            workspace_settings_path.parent.mkdir(exist_ok=True)
            with open(workspace_settings_path, 'w') as f:
                f.write(stringified_config)
            print('Successfully overwrote "{}".'.format(workspace_settings_path))
        else:
            print('Aborted overwriting "{}".'.format(workspace_settings_path))
    else:
        print(stringified_config)


def get_editor_config() -> dict:
    return {
        # Python Extension
        # https://marketplace.visualstudio.com/items?itemName=ms-python.python
        # ==============================================================================

        # General Settings
        # https://code.visualstudio.com/docs/python/settings-reference#_general-settings
        # ------------------------------------------------------------------------------
        # For FreeCAD auto-completion to pick-up .env file
        'python.envFile': '${workspaceFolder}/.env',

        # Formatting
        # https://code.visualstudio.com/docs/python/editing#_formatting
        # ------------------------------------------------------------------------------
        'python.formatting.provider': 'autopep8',

        # Linting
        # https://code.visualstudio.com/docs/python/linting
        # ------------------------------------------------------------------------------
        'python.linting.enabled': True,
        'python.linting.lintOnSave': True,
        'python.linting.pylintEnabled': False,
        'python.linting.flake8Enabled': True,
        'python.linting.flake8Args': [
            '--config {}'.format(path('.flake8'))
        ],
        # display flake8 warnings as errors
        'python.linting.flake8CategorySeverity.W': 'Error',
        'python.linting.mypyEnabled': True,
        # ------------------------------------------------------------------------------

        # Python Docstring Generator Extension
        # https://marketplace.visualstudio.com/items?itemName=njpwerner.autodocstring
        # ==============================================================================
        'autoDocstring.docstringFormat': 'sphinx',
        'autoDocstring.customTemplatePath': path('.mustache')
        # ------------------------------------------------------------------------------
    }


def path(filename: str) -> str:
    current_dir = Path(os.path.dirname(os.path.realpath(__file__)))
    return str(current_dir.joinpath(filename).resolve())


def get_vs_code_workspace_settings(root_of_git_repository: str) -> Optional[dict]:
    path = get_vs_code_workspace_settings_path(root_of_git_repository)
    if not path.exists():
        return None
    with path.open() as f:
        settings = json.load(f)
    return settings


def get_vs_code_workspace_settings_path(root_of_git_repository: str) -> Path:
    """Get the path to VS Code workspace settings.

    See Also:
        https://code.visualstudio.com/docs/getstarted/settings#_settings-file-locations
    """
    return Path(root_of_git_repository) / '.vscode' / 'settings.json'


def query_yes_no(question, additional_pre_information: str = None, default: str = 'yes'):
    """Ask a yes/no question via raw_input() and return their answer.

    "question" is a string that is presented to the user.
    "default" is the presumed answer if the user just hits <Enter>.
    It must be "yes" (the default), "no" or None (meaning
    an answer is required of the user).

    The "answer" return value is True for "yes" or False for "no".
    """
    valid = {'yes': True, 'y': True, 'ye': True,
             'no': False, 'n': False}
    if default is None:
        prompt = ' [y/n] '
    elif default == 'yes':
        prompt = ' [Y/n] '
    elif default == 'no':
        prompt = ' [y/N] '
    else:
        raise ValueError('Invalid default answer "%s".' % default)

    while True:
        if additional_pre_information:
            print(additional_pre_information)
        print(question + prompt)
        choice = input().lower()
        if default is not None and choice == '':
            return valid[default]
        elif choice in valid:
            return valid[choice]
        else:
            print('Please respond with "yes" or "no" (or "y" or "n").\n')



from FreeCAD import Placement, Vector

from {{ base_package }}.model import {{ model }}


def create_{{ name }}(document, name):
    """
    Creates a part feature object with the given name,
    and adds it to the document.
    """
    obj = document.addObject('Part::FeaturePython', name)
    {{ model }}(obj)
    obj.ViewObject.Proxy = 0  # Mandatory unless ViewProvider is coded
    return obj



import argparse
import os
from typing import Tuple

from ._version import __version__
from .execute_command import execute_command
from .find_base_package import find_base_package, find_root_of_git_repository
from .handle_browse_command import handle_browse_command
from .handle_build_command import handle_build_command
from .handle_docs_command import handle_docs_command
from .handle_editor_config_command import handle_editor_config_command
from .handle_lint_command import handle_lint_command
from .handle_make_component_command import handle_make_component_command
from .handle_make_workbench_command import handle_make_workbench_command
from .handle_test_command import handle_test_command


def main() -> None:
    command, args = _parse_command()
    if command == 'make' and (args['make_command'] == 'workbench' or args['make_command'] == 'wb'):
        machine_display_name = args['machine_display_name']
        handle_make_workbench_command(machine_display_name)
    elif command == 'test' or command == 'docs' or command == 'make':
        root_of_git_repository = find_root_of_git_repository()
        if root_of_git_repository is None:
            return None
        base_package = find_base_package()
        if base_package is None:
            return None
        if command == 'test':
            handle_test_command(base_package,
                                root_of_git_repository,
                                with_coverage=args['coverage'])
        elif command == 'docs':
            if args['docs_command'] == 'screenshot' or args['docs_command'] == 'ss':
                screenshot_script = os.path.join(os.path.dirname(
                    os.path.abspath(__file__)), 'part_screenshot.py')
                execute_command('freecad -c {}'.format(screenshot_script))
            else:
                handle_docs_command(base_package, root_of_git_repository)
        elif command == 'make':
            args_copy = args.copy()
            make_subcommand = args_copy.pop('make_command').replace('-', '_')
            try:
                make_subcommand = {
                    'pf': 'part_feature'
                }[make_subcommand]
            except KeyError:
                pass
            name = args_copy.pop('name')
            handle_make_component_command(base_package,
                                          root_of_git_repository,
                                          make_subcommand,
                                          name,
                                          args_copy)
    elif command == 'browse' or command == 'lint' or command == 'editor-config':
        root_of_git_repository = find_root_of_git_repository()
        if root_of_git_repository is None:
            return None
        if command == 'browse':
            browse_subcommand = args['browse_command']
            handle_browse_command(root_of_git_repository, browse_subcommand)
        elif command == 'lint':
            should_fix = args['fix']
            handle_lint_command(root_of_git_repository, should_fix)
        elif command == 'editor-config':
            handle_editor_config_command(
                root_of_git_repository, args['merge_workspace_settings'], args['overwrite_workspace_settings'])
    elif command == 'build':
        handle_build_command()


def _parse_command() -> Tuple[str, dict]:
    parser = argparse.ArgumentParser(
        description='A collection commands for OSE workbench development.',
        usage='osewb <command> [<args>]\n')
    parser.add_argument('--version', action='version', version=__version__)
    subparsers = parser.add_subparsers(title='Commands',
                                       dest='command',
                                       required=True)
    test_parser = subparsers.add_parser('test',
                                        help='Run tests in workbench',
                                        usage='osewb test')
    test_parser.add_argument('-c', '--coverage',
                             action='store_true',
                             help='Run tests with coverage, and generate report')
    lint_parser = subparsers.add_parser('lint',
                                        help='Lint code',
                                        usage='osewb lint')
    lint_parser.add_argument('-f', '--fix',
                             action='store_true',
                             help='Attempt to automatically fix linter issues')
    docs_parser = subparsers.add_parser('docs',
                                        help='Make documentation',
                                        usage='osewb docs [command]')
    docs_subparser = docs_parser.add_subparsers(title='Commands',
                                                dest='docs_command',
                                                required=False)
    docs_subparser.add_parser('screenshot',
                              help='Take screenshots of parts for documentation.',
                              usage='osewb docs screenshot',
                              aliases=['ss'])
    make_parser = subparsers.add_parser('make',
                                        help='Commands for making new code',
                                        usage='osewb make <command>',
                                        aliases=['mk'])
    make_subparser = make_parser.add_subparsers(title='Commands',
                                                dest='make_command',
                                                required=True)
    workbench_subparser = make_subparser.add_parser('workbench',
                                                    help='Make Workbench',
                                                    usage='osewb make workbench <machine_display_name>',
                                                    aliases=['wb'])
    workbench_subparser.add_argument('machine_display_name',
                                     type=str,
                                     help='Name of machine in title-case. Surround in double-quotes if name contains spaces (e.g. "CEB Brick Press")')
    part_subparser = make_subparser.add_parser('part',
                                               help='Make Part class',
                                               usage='osewb make part <name>')
    part_subparser.add_argument(
        'name', help='Name for the part class in pascal or upper camel-case (e.g. MyBox).')
    model_subparser = make_subparser.add_parser('model',
                                                help='Make Model class',
                                                usage='osewb make model <name>')
    model_subparser.add_argument(
        'name', help='Name for the model class in pascal or upper camel-case (e.g. MyBox).')
    model_subparser.add_argument('-p', '--part',
                                 action='store_true',
                                 help='Make part class as well.')
    part_feature_subparser = make_subparser.add_parser('part-feature',
                                                       help='Make Part Feature creation function',
                                                       usage='osewb make part-feature <name>',
                                                       aliases=['pf'])
    part_feature_subparser.add_argument(
        'name', help='Name for the part feature creation function in all-lower snake-case (e.g. my_box).')
    command_subparser = make_subparser.add_parser('command',
                                                  help='Make Command class',
                                                  usage='osewb make command <name>')
    command_subparser.add_argument(
        'name', help='Name for the command class in pascal or upper camel-case (e.g. MyBoxCommand).')
    browse_parser = subparsers.add_parser('browse',
                                          help='Commands for opening documents in a web browser',
                                          usage='osewb browse <command>',
                                          aliases=['br'])
    browse_subparser = browse_parser.add_subparsers(title='Commands',
                                                    dest='browse_command',
                                                    required=True)
    browse_subparser.add_parser('docs',
                                help='Opens docs in web browser',
                                usage='osewb browse docs')
    browse_subparser.add_parser('coverage',
                                help='Opens coverage report in web browser',
                                usage='osewb browse coverage',
                                aliases=['cov'])
    build_parser = subparsers.add_parser('build',
                                         help='Build a workbench',
                                         usage='osewb build',
                                         aliases=['bld'])
    editor_config_parser = subparsers.add_parser('editor-config',
                                                 help='Output config for VS Code editor.',
                                                 usage='osewb editor-config',
                                                 aliases=['ec'])
    editor_config_parser.add_argument('-m', '--merge-workspace-settings',
                                      action='store_true',
                                      help='Merge VS Code workspace settings.')
    editor_config_parser.add_argument('-o', '--overwrite-workspace-settings',
                                      action='store_true',
                                      help='Overwrite VS Code workspace settings.')
    args = vars(parser.parse_args())
    command = args.pop('command')
    # TODO: Map short-alias of sub-commands to full-command name before returning!
    #       For example, ss -> screenshot or wb -> workbench..
    return map_potential_command_alias(command), args


def map_potential_command_alias(command: str):
    try:
        return {
            'mk': 'make',
            'br': 'browse',
            'bld': 'build',
            'ec': 'editor-config'
        }[command]
    except KeyError:
        return command


if __name__ == '__main__':
    main()



"""Utility script to automatically create thumbnail screenshots of parts.

Run with freecad -c part_screenshot.py when conda environment is activated.
"""
import importlib
import inspect
import os
from pathlib import Path

import FreeCAD as App
import FreeCADGui as Gui
import Part
from osewb.find_base_package import find_base_package


def main():
    base_package = find_base_package()
    if not base_package:
        exit(0)

    part_package = importlib.import_module(base_package + '.part')

    def is_part_class(x) -> bool:
        return inspect.isclass(x) and not inspect.isbuiltin(x)

    members = inspect.getmembers(part_package, is_part_class)

    # Setup Gui
    Gui.showMainWindow()
    main_window = Gui.getMainWindow()
    main_window.hide()

    # Document needs to be created after Gui is setup
    document = App.newDocument()

    screenshot_path = Path(os.path.join('docs', '_static', 'screenshot'))
    screenshot_path.mkdir(parents=True, exist_ok=True)

    active_view = Gui.getDocument(document.Name).activeView()
    active_view.setCameraType('Orthographic')
    active_view.setAnimationEnabled(False)

    # It's important not to use Qt's QGLFramebufferObject because it crashes when no GUI is shown
    # but Qt's QGLPixelBuffer still works.
    parameter = App.ParamGet("User parameter:BaseApp")
    group = parameter.GetGroup("Preferences/Document")
    group.SetBool("DisablePBuffers", False)

    for (name, part) in members:
        made_part = part.make()
        Part.show(made_part)

        active_view.viewIsometric()
        active_view.fitAll()

        image_name = str(screenshot_path.joinpath('{}.png'.format(name)))
        print('Saving image {}'.format(image_name))
        active_view.saveImage(image_name, 150, 150, 'Transparent')

        document.removeObject('Shape')

    App.closeDocument(document.Name)
    exit(0)


if __name__ == '__main__':
    main()
